#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

select_boot_setup() {
    # choose bootloader and by extension mountpoint (if needed)
    tput civis
    BOOTLDR="$(dialog --cr-wrap --stdout --backtitle "$BT" \
        --title " $_PrepMount " --menu "$_MntBootBody" 0 0 0 ${BOOTLDRS[$SYS]})"
    [[ $? != 0 || $BOOTLDR == "" ]] && return 1

    if [[ $SYS == 'BIOS' && $BOOTLDR == 'grub' && $BOOT_DEVICE == "" ]]; then
        # grub on BIOS needs an install device, NOT partition eg. /dev/sda
        select_device 'boot' || return 1
    fi

    if [[ $BOOTLDR == 'systemd-boot' ]]; then
        FILES[9]="/boot/loader/entries/$DIST.conf"
    elif [[ $BOOTLDR == 'syslinux' ]]; then
        if [[ $SYS == 'BIOS' ]]; then
            FILES[9]="/boot/syslinux/syslinux.cfg"
            BCMDS[$BOOTLDR]="$(dialog --cr-wrap --stdout --backtitle "$BT" \
                --title " $_InstSysTitle " --menu "$_InstSysBody" 0 0 0 \
                "syslinux-install_update -iam" "Install to MBR (Master Boot Record)" \
                "syslinux-install_update -i" "Install to root partition (/)")"
            [[ $? != 0 || ${BCMDS[$BOOTLDR]} == "" ]] && return 1
        else
            FILES[9]="/boot/EFI/syslinux/syslinux.cfg"
            BCMDS[syslinux]="efibootmgr -c -d $BOOT_DEVICE -p $BOOT_PART_NUM -l /EFI/syslinux/syslinux.efi -L $DIST"
        fi
    else
        FILES[9]="/etc/default/grub"

        if [[ $SYS == 'UEFI' ]]; then
            local ttype
            [[ $IS_64BIT != true ]] && ttype='i386-efi' || ttype='x86_64-efi'
            BCMDS[grub]="${BCMDS[grub]} --target=$ttype --bootloader-id=$DIST"
        else
            BCMDS[grub]="${BCMDS[grub]} --target=i386-pc $BOOT_DEVICE"
        fi

        BCMDS[grub]="${BCMDS[grub]} && grub-mkconfig -o /boot/grub/grub.cfg"
    fi

    return 0
}

shim_secure_boot() {
    # still a W.I.P
    local shim_file="shim64.efi"
    [[ $IS_64BIT != true ]] && shim_file="shim.efi"
    efibootmgr -c -w -L $DIST -d $BOOT_DEVICE -p $BOOT_PART_NUM -l ${MNT}${BMNTS[$SYS-$BOOTLDR]}/$shim_file
    return 0
}

uefi_boot_fallback() {
    # some UEFI firmware requires a directory in the ESP and a generic bootx64.efi
    # see:  https://wiki.archlinux.org/index.php/GRUB#UEFI
    # also: https://wiki.archlinux.org/index.php/syslinux#UEFI_Systems

    local esp="${MNT}${BMNTS[$SYS-$BOOTLDR]}"

    local default
    default="$(find $esp/EFI/ -maxdepth 1 -mindepth 1 -name '[Bb][oO][oO][tT]' -type d)"
    [[ $default ]] && default="$(basename $default)" || default="boot"

    # directory exists: remove everything inside, otherwise mkdir it
    [[ -d $esp/EFI/$default ]] && rm -rf $esp/EFI/$default/* || mkdir -p $esp/EFI/$default

    # copy efi stub to the new default location
    if [[ $BOOTLDR == 'syslinux' ]]; then
        # syslinux requires the whole boot setup be copied, not just the stub
        cp -rf $esp/EFI/syslinux/* $esp/EFI/$default/
        cp -f $esp/EFI/syslinux/syslinux.efi $esp/EFI/$default/bootx64.efi
    elif [[ $BOOTLDR == 'grub' ]]; then
        # grub uses different names for the stub depending on architecture
        if [[ $IS_64BIT != true ]]; then
            local grub_file="grubia32.efi"
            local boot_file="bootia32.efi"
        else
            local grub_file="grubx64.efi"
            local boot_file="bootx64.efi"
        fi
        cp -f $esp/EFI/$DIST/$grub_file $esp/EFI/$default/$boot_file
    fi
    return 0
}

prep_for_grub() {
    local cfg="$MNT/etc/default/grub"

    sed -i "s/GRUB_DISTRIBUTOR=.*/GRUB_DISTRIBUTOR=\"${DIST}\"/g;
            s/GRUB_CMDLINE_LINUX_DEFAULT=.*/GRUB_CMDLINE_LINUX_DEFAULT=\"\"/g" $cfg

    if [[ $LUKS -eq 1 ]]; then
        sed -i "s~#GRUB_ENABLE_CRYPTODISK~GRUB_ENABLE_CRYPTODISK~g;
                s~GRUB_CMDLINE_LINUX=.*~GRUB_CMDLINE_LINUX=\"${LUKS_DEV}\"~g" $cfg
    fi

    if [[ $SYS == 'BIOS' && $LVM -eq 1 && $SEPERATE_BOOT == false ]]; then
        sed -i "s/GRUB_PRELOAD_MODULES=.*/GRUB_PRELOAD_MODULES=\"lvm\"/g" $cfg
    fi

    return 0
}

prep_for_systemd-boot() {
    # no LVM then systemd-boot uses PARTUUID
    ! [[ $ROOT_PART =~ /dev/mapper ]] && ROOT_PART_ID="PART$ROOT_PART_ID"

    # create the boot entry configs
    mkdir -p $MNT/boot/loader/entries
    cat > $MNT/boot/loader/loader.conf << EOF
default  $DIST
timeout  5
editor   no
EOF
    cat > $MNT/boot/loader/entries/${DIST}.conf << EOF
title   $DIST Linux
linux   /vmlinuz-${KERNEL}$([[ $UCODE ]] && echo -en "\ninitrd  /$UCODE")
initrd  /initramfs-$KERNEL.img
options root=$ROOT_PART_ID $([[ $LUKS_DEV ]] && echo -n "$LUKS_DEV")rw
EOF
    # add pacman hook to update the bootloader when systemd receives an update
    mkdir -p $MNT/pacman.d/hooks
    cat > $MNT/pacman.d/hooks/systemd-boot.hook << EOF
[Trigger]
Type = Package
Operation = Upgrade
Target = systemd

[Action]
Description = Updating systemd-boot
When = PostTransaction
Exec = /usr/bin/bootctl update
EOF
    # systemd-boot requires this before running bootctl
    systemd-machine-id-setup --root="$MNT" >/dev/null 2>&1
    return 0
}

prep_for_syslinux() {
    if [[ $SYS == 'UEFI' ]]; then
        local cfgdir="$MNT/boot/EFI/syslinux"
        if [[ $IS_64BIT == true ]]; then
            local cfgsrcdir="/usr/lib/syslinux/efi64/"
        else
            local cfgsrcdir="/usr/lib/syslinux/efi32/"
        fi
    else
        local cfgdir="$MNT/boot/syslinux"
        local cfgsrcdir="/usr/lib/syslinux/bios/"
    fi

    mkdir -p $cfgdir
    cp -r $cfgsrcdir $cfgdir/
    cat > $cfgdir/syslinux.cfg << EOF
UI menu.c32
PROMPT 0

MENU TITLE $DIST Syslinux Boot Menu
TIMEOUT 50
DEFAULT $DIST

LABEL $DIST
MENU LABEL $DIST Linux
LINUX ../vmlinuz-$KERNEL
APPEND root=$ROOT_PART_ID $([[ $LUKS_DEV ]] && echo -n "$LUKS_DEV ")rw
INITRD ../initramfs-$KERNEL.img
$([[ $UCODE ]] && echo -en "\nINITRD ../$UCODE")

LABEL ${DIST}fallback
MENU LABEL $DIST Linux Fallback
LINUX ../vmlinuz-$KERNEL
APPEND root=$ROOT_PART_ID $([[ $LUKS_DEV ]] && echo -n "$LUKS_DEV ")rw
INITRD ../initramfs-$KERNEL-fallback.img
$([[ $UCODE ]] && echo -en "\nINITRD ../$UCODE")
EOF
    return 0
}

install_bootloader() {
    # not an LVM we can use the UUID for booting otherwise use the partition label
    if ! [[ $ROOT_PART =~ /dev/mapper ]]; then
        ROOT_PART_ID="UUID=$(blkid -s PARTUUID $ROOT_PART | sed 's/.*=//g; s/"//g')"
    else
        ROOT_PART_ID="$ROOT_PART"
    fi

    # remove old UEFI boot entries
    [[ $SYS == 'UEFI' ]] && find $MNT${BMNTS[$SYS-$BOOTLDR]}/EFI/ -maxdepth 1 -mindepth 1 \
        -name '[aA][rR][cC][hH][lL]abs' -type d -exec rm -rf '{}' \; >/dev/null 2>&1

    # sets up the bootloader configs
    prep_for_$BOOTLDR

    # run the bootloader command
    chroot_cmd "${BCMDS[$BOOTLDR]}" 2>$ERR
    check_for_errors "${BCMDS[$BOOTLDR]}"

    # copy efi stub to generic catch all
    [[ $SYS == 'UEFI' && $BOOTLDR =~ (grub|syslinux) ]] && uefi_boot_fallback
    return 0
}
